package JavaScript::RPC;

our $VERSION = '0.2';

=head1 NAME

JavaScript::RPC - (DEPRECATED) Remote procedure calls from JavaScript

=head1 DEPRECATION NOTICE

Brent Ashley, the author of JSRS, has stated that he has no plans to update
his library. JavaScript RPC, more commonly known as "AJAX", has seen a
rise in popularity and therefore many new libraries have been released.
You should consider those alternatives before trying to use this module and
the JSRS library.

=head1 SEE ALSO

=over 4 

=item * Dojo JavaScript Toolkit - L<http://dojotoolkit.org/>

=item * MochiKit - L<http://mochikit.com/>

=item * Prototype JavaScript Framework - L<http://prototype.conio.net/>

=back

=head1 AUTHOR

=over 4 

=item * Brian Cassidy E<lt>bricas@cpan.orgE<gt>

=back

=head1 COPYRIGHT AND LICENSE

Copyright 2006 by Brian Cassidy

This library is free software; you can redistribute it and/or modify
it under the same terms as Perl itself. 

=cut

1;
